@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -52,7 +52,6 @@ package com.openexchange.push.udp;
 import java.net.DatagramPacket;
 import java.net.InetAddress;
 import java.net.MulticastSocket;
-import java.net.UnknownHostException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.timer.ScheduledTimerTask;
@@ -62,11 +61,11 @@ import com.openexchange.tools.encoding.Charsets;
 /**
  * PushOutputQueue
  * 
- * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
+ * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  */
 public class PushDiscoverySender implements Runnable {
 
-    private static final Log LOG = LogFactory.getLog(PushDiscoverySender.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(PushDiscoverySender.class));
 
     private final int multicastPort;
 
@@ -82,19 +81,12 @@ public class PushDiscoverySender implements Runnable {
 
     private ScheduledTimerTask task;
 
-    public PushDiscoverySender(final PushConfig pushConfigInterface) {
+    public PushDiscoverySender(final PushConfiguration pushConfigInterface) {
         super();
 
         InetAddress hostname = pushConfigInterface.getHostName();
-        if (hostname == null) {
-            try {
-                hostname = InetAddress.getLocalHost();
-            } catch (UnknownHostException exc) {
-                LOG.warn("unable to resolv local address", exc);
-            }
-        }
 
-        discoveryData = String.valueOf(PushRequest.REMOTE_HOST_REGISTER) + '\1' + (hostname == null ? "localhost" : hostname.getHostName())
+        discoveryData = String.valueOf(PushRequest.REMOTE_HOST_REGISTER) + '\1' + hostname.getCanonicalHostName()
             + '\1' + String.valueOf(pushConfigInterface.getRegisterPort());
         packetData = String.valueOf(PushRequest.MAGIC) + '\1' + String.valueOf(discoveryData.length()) + '\1' + discoveryData;
         packetBytes = Charsets.getBytes(packetData, Charsets.US_ASCII);
@@ -107,15 +99,15 @@ public class PushDiscoverySender implements Runnable {
     public void run() {
         try {
             LOG.debug("Sending multicast discovery package: \"" + packetData + "\".");
-            MulticastSocket multicastSocket = PushMulticastSocket.getPushMulticastSocket();
-            DatagramPacket datagramPacket = new DatagramPacket(packetBytes, packetBytes.length, multicastAddress, multicastPort);
+            final MulticastSocket multicastSocket = PushMulticastSocket.getPushMulticastSocket();
+            final DatagramPacket datagramPacket = new DatagramPacket(packetBytes, packetBytes.length, multicastAddress, multicastPort);
             multicastSocket.send(datagramPacket);
-        } catch (Exception e) {
+        } catch (final Exception e) {
             LOG.error(e.getMessage(), e);
         }
     }
 
-    public void startSender(TimerService service) {
+    public void startSender(final TimerService service) {
         task = service.scheduleAtFixedRate(this, 0, remoteHostFresh);
     }
 
