@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,9 +49,8 @@
 
 package com.openexchange.push.udp;
 
-import java.net.DatagramPacket;
-import java.net.DatagramSocket;
-import java.net.InetAddress;
+import static com.openexchange.push.udp.PushChannels.ChannelType.EXTERNAL;
+import static com.openexchange.push.udp.PushChannels.ChannelType.INTERNAL;
 import java.util.Dictionary;
 import java.util.HashMap;
 import java.util.Hashtable;
@@ -73,7 +72,7 @@ import com.openexchange.tools.StringCollection;
 /**
  * {@link PushOutputQueue} - Holds main queue containing objects to deliver.
  * 
- * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
+ * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  */
 public class PushOutputQueue implements Runnable {
 
@@ -85,11 +84,9 @@ public class PushOutputQueue implements Runnable {
 
     private static boolean isInit;
 
-    private static final Log LOG = LogFactory.getLog(PushOutputQueue.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(PushOutputQueue.class));
 
-    private static final boolean DEBUG = LOG.isDebugEnabled();
-
-    private static PushConfig pushConfigInterface;
+    private static PushConfiguration pushConfigInterface;
 
     private static DelayQueue<PushDelayedObject> queue = new DelayQueue<PushDelayedObject>();
 
@@ -97,35 +94,40 @@ public class PushOutputQueue implements Runnable {
 
     private static int remoteHostTimeOut = 3600000;
 
+    private static PushChannels channels;
+    
     /**
      * Adds specified push object to queue for delivery.
-     * 
+     *
      * @param pushObject The push object to deliver
      * @throws EventException If an event exception occurs
      */
     public static void add(final PushObject pushObject) throws EventException {
-        if (DEBUG) {
+        if (LOG.isDebugEnabled()) {
             LOG.debug("add PushObject: " + pushObject);
         }
-
         if (!isEnabled) {
             return;
         }
-
         if (!isInit) {
             throw new EventException("PushOutputQueue not initialisiert!");
         }
-
-        final PushDelayedObject pushDelayedObject;
-        if (existingPushObjects.containsKey(pushObject)) {
-            pushDelayedObject = existingPushObjects.get(pushObject);
+        PushDelayedObject pushDelayedObject = existingPushObjects.get(pushObject);
+        if (Types.EMAIL == pushObject.getModule()) {
+            // No delay for emails. Push them immediately to the client. EMails can not be changed and every new push event indicates a new
+            // email in the INBOX which needs to be transfered to the client.
+            // Do not put into existingPushObjects because we do not need to find it again.
+            queue.add(new PushDelayedObject(0, pushObject));
+        } else if (null != pushDelayedObject) {
+            // Delay push for same PIM objects folder because of further changes in the same folder. The maximum for this further delaying
+            // is done in the {@link PushDelayedObject}.
+            // If existingPushObjects finds a {@link PushDelayedObject} it indicates the same folder and the same affected users.
             pushDelayedObject.updateTime();
         } else {
             pushDelayedObject = new PushDelayedObject(delay, pushObject);
+            existingPushObjects.put(pushObject, pushDelayedObject);
+            queue.add(pushDelayedObject);
         }
-
-        existingPushObjects.put(pushObject, pushDelayedObject);
-        queue.add(pushDelayedObject);
     }
 
     /**
@@ -146,7 +148,7 @@ public class PushOutputQueue implements Runnable {
      * @throws EventException If an event exception occurs
      */
     public static void add(final RegisterObject registerObject, final boolean noDelay) throws EventException {
-        if (DEBUG) {
+        if (LOG.isDebugEnabled()) {
             LOG.debug("add RegisterObject: " + registerObject);
         }
 
@@ -189,17 +191,22 @@ public class PushOutputQueue implements Runnable {
         final int users[] = pushObject.getUsers();
         final int contextId = pushObject.getContextId();
         final int folderId = pushObject.getFolderId();
-        for (int a = 0; a < users.length; a++) {
-
-            if (RegisterHandler.isRegistered(users[a], contextId)) {
-                final RegisterObject registerObj = RegisterHandler.getRegisterObject(users[a], contextId);
+        {
+            final byte[] bytes;
+            {
                 final StringBuilder sb = new StringBuilder();
                 sb.append(folderId);
                 sb.append('\1');
-                try {
-                    makeAndSendPackage(sb.toString().getBytes(), registerObj.getHostAddress(), registerObj.getPort());
-                } catch (final Exception exc) {
-                    LOG.error("createPushPackage", exc);
+                bytes = sb.toString().getBytes();
+            }
+            for (final int user : users) {
+                if (RegisterHandler.isRegistered(user, contextId)) {
+                    final RegisterObject registerObj = RegisterHandler.getRegisterObject(user, contextId);
+                    try {
+                        channels.makeAndSendPackage(bytes, registerObj.getHostAddress(), registerObj.getPort(), EXTERNAL);
+                    } catch (final Exception exc) {
+                        LOG.error("createPushPackage", exc);
+                    }
                 }
             }
         }
@@ -228,9 +235,9 @@ public class PushOutputQueue implements Runnable {
                     topic = "com/openexchange/remote/foldercontentchanged";
                 }
 
-                for (int a = 0; a < users.length; a++) {
-                    final RemoteEvent remoteEvent = factoryService.newRemoteEvent(folderId, users[a], contextId, action, module, timestamp);
-                    final Dictionary<String, RemoteEvent> ht = new Hashtable<String, RemoteEvent>();
+                for (final int user : users) {
+                    final RemoteEvent remoteEvent = factoryService.newRemoteEvent(folderId, user, contextId, action, module, timestamp);
+                    final Dictionary<String, RemoteEvent> ht = new Hashtable<String, RemoteEvent>(1);
                     ht.put(RemoteEvent.EVENT_KEY, remoteEvent);
                     eventAdmin.postEvent(new Event(topic, ht));
                 }
@@ -269,7 +276,7 @@ public class PushOutputQueue implements Runnable {
                     final byte b[] = sb.toString().getBytes();
 
                     if (System.currentTimeMillis() <= (remoteHostObject.getTimer().getTime() + remoteHostTimeOut)) {
-                        makeAndSendPackage(b, remoteHostObject.getHost(), remoteHostObject.getPort());
+                        channels.makeAndSendPackage(b, remoteHostObject.getHost(), remoteHostObject.getPort(), INTERNAL);
                     } else {
                         if (LOG.isTraceEnabled()) {
                             LOG.trace("remote host object is timed out");
@@ -294,7 +301,7 @@ public class PushOutputQueue implements Runnable {
             final StringBuilder sb = new StringBuilder();
             sb.append("OK\1");
             try {
-                makeAndSendPackage(sb.toString().getBytes(), registerObject.getHostAddress(), registerObject.getPort());
+                channels.makeAndSendPackage(sb.toString().getBytes(), registerObject.getHostAddress(), registerObject.getPort(), EXTERNAL);
             } catch (final Exception exc) {
                 LOG.error("createRegisterPackage", exc);
             }
@@ -332,7 +339,7 @@ public class PushOutputQueue implements Runnable {
 
                     final byte b[] = sb.toString().getBytes();
                     if (System.currentTimeMillis() <= (remoteHostObject.getTimer().getTime() + remoteHostTimeOut)) {
-                        makeAndSendPackage(b, remoteHostObject.getHost(), remoteHostObject.getPort());
+                        channels.makeAndSendPackage(b, remoteHostObject.getHost(), remoteHostObject.getPort(), INTERNAL);
                     } else {
                         if (LOG.isTraceEnabled()) {
                             LOG.trace("remote host object is timed out");
@@ -355,7 +362,7 @@ public class PushOutputQueue implements Runnable {
         final AbstractPushObject abstractPushObject = pushDelayedObject.getPushObject();
 
         if (abstractPushObject instanceof PushObject) {
-            if (DEBUG) {
+            if (LOG.isDebugEnabled()) {
                 LOG.debug("Send Push Object");
             }
 
@@ -364,7 +371,7 @@ public class PushOutputQueue implements Runnable {
 
             createAndDeliverPushPackage(pushObject);
         } else if (abstractPushObject instanceof RegisterObject) {
-            if (DEBUG) {
+            if (LOG.isDebugEnabled()) {
                 LOG.debug("Send Register Object");
             }
 
@@ -372,19 +379,11 @@ public class PushOutputQueue implements Runnable {
         }
     }
 
-    private static void makeAndSendPackage(final byte[] b, final InetAddress host, final int port) throws Exception {
-        final DatagramSocket datagramSocket = PushSocket.getPushDatagramSocket();
-        final DatagramPacket datagramPackage = new DatagramPacket(b, b.length, host, port);
-        datagramSocket.send(datagramPackage);
-    }
-
-    private static void makeAndSendPackage(final byte[] b, final String host, final int port) throws Exception {
-        makeAndSendPackage(b, InetAddress.getByName(host), port);
-    }
+  
 
     private boolean isRunning = false;
 
-    public PushOutputQueue(final PushConfig pushConfigInterface) {
+    public PushOutputQueue(final PushConfiguration pushConfigInterface, final PushChannels channels2) {
         PushOutputQueue.pushConfigInterface = pushConfigInterface;
 
         remoteHost = pushConfigInterface.getRemoteHost();
@@ -403,6 +402,8 @@ public class PushOutputQueue implements Runnable {
             isEnabled = true;
 
             isRunning = true;
+            
+            channels = channels2;
 
             final Thread th = new Thread(this);
             th.setName(this.getClass().getName());
@@ -422,7 +423,7 @@ public class PushOutputQueue implements Runnable {
 
     public void run() {
         while (isRunning) {
-            if (DEBUG) {
+            if (LOG.isDebugEnabled()) {
                 LOG.debug("get push objects from queue: " + queue.size());
             }
 
