@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,7 +54,7 @@ import java.util.Date;
 /**
  * {@link RegisterObject} - Represents a registered client waiting for remote events.
  * 
- * @author <a href="mailto:sebastian.kauss@netline-is.de">Sebastian Kauss</a>
+ * @author <a href="mailto:sebastian.kauss@open-xchange.com">Sebastian Kauss</a>
  */
 public class RegisterObject extends AbstractPushObject {
 
@@ -69,11 +69,10 @@ public class RegisterObject extends AbstractPushObject {
     private final Date timestamp;
 
     public RegisterObject(final int userId, final int contextId, final String hostAddress, final int port, final boolean isSync) {
+        super(contextId, isSync);
         this.userId = userId;
-        this.contextId = contextId;
         this.hostAddress = hostAddress;
         this.port = port;
-        this.remote = isSync;
         timestamp = new Date();
     }
 
@@ -97,23 +96,9 @@ public class RegisterObject extends AbstractPushObject {
         return timestamp;
     }
 
-    @Override
-    public int hashCode() {
-        return toString().hashCode();
-    }
-
-    @Override
-    public boolean equals(final Object obj) {
-        if (obj == null) {
-            return false;
-        }
-
-        return (hashCode() == obj.hashCode());
-    }
-
     @Override
     public String toString() {
-        return new StringBuilder().append("USER_ID=").append(userId).append(",CONTEXT_ID=").append(contextId).append(",ADDRESS=").append(
+        return new StringBuilder().append("USER_ID=").append(userId).append(",CONTEXT_ID=").append(getContextId()).append(",ADDRESS=").append(
             hostAddress).append(",PORT").append(port).toString();
     }
 }
